#ifndef _EAN_13_h
#define _EAN_13_h

#include <array> // 'std::to_array' from C++20 (https://en.cppreference.com/w/cpp/container/array/to_array)
#include <bitset>
#include <initializer_list>
#include <stdexcept>
#include <string_view> // 'std::hash' for string view objects...
#include <unordered_map>

class EAN_13 { // 'EAN_13' objects act as keys in unordered maps...
private:
	// https://www.codesdope.com/cpp-stdarray (warning: 'std::array 'size is defined at compilation time!)
	std::array<char, 13> _representation{}; // '{}' => The 13 digits are set to '\0' character...
public:
	EAN_13(const std::initializer_list<char>&);
	// Key computation (hash) requires contextual definition of '==' operator:
	bool operator==(const EAN_13&) const;
	std::string_view toString() const; // 'std::string_view' from C++17 (https://www.safetica.com/blog/do-you-know-std-string_view-and-how-to-use-it)
};

class _EAN_13_key {
public:
	// Key computation (hash) requires contextual definition of '()' operator:
	std::size_t operator()(const EAN_13&) const noexcept;
};

class EAN_13_database {
private:
	// Third parameter is the hash function:
	std::unordered_map<EAN_13, std::bitset<4096>, _EAN_13_key> _representation; // As an illustration, values may be images of 16x16 grayed pixels without alpha...
public:
	EAN_13_database(const std::initializer_list<EAN_13>&);
	inline std::size_t size() const { return _representation.size(); }
};

#endif
