#include <cassert>
#include <cstring> // '::strncpy'

#include <iostream>

#include "EAN_13.h"

// 'std::array' has no constructor; '_representation(digits)' won't work...
EAN_13::EAN_13(const std::initializer_list<char>& digits) /*: _representation(digits)*/ {
	std::array<char, 13>::iterator j = _representation.begin();
	for (std::initializer_list<char>::const_iterator i = digits.begin(); !(i == digits.end()); i++, j++)
		*j = *i;
}
bool EAN_13::operator==(const EAN_13& product) const {
	return _representation == product._representation; // 'std::array' supports '==' (https://en.cppreference.com/w/cpp/container/array/operator_cmp)
}
std::string_view EAN_13::toString() const {
	const char* data = this->_representation.data(); // Primitive underlying array *WITHOUT* '\0'!
	/** Back to middle age */
	char buffer[13 + 1];
	buffer[13] = '\0';
	::strncpy(buffer, data, 13);
	/** End of "back to middle age" */

	// std::string_view sv{ data };
	// sv.remove_suffix(sv.size() - 13);

	std::string_view sv(buffer); // 'std::string_view' avoids copy here...
	return sv;
}

std::size_t _EAN_13_key::operator()(const EAN_13& product) const noexcept {
	// Robustness invites us to reuse key computation based on string view objects:
	return std::hash<std::string_view>{}(product.toString());
}

EAN_13_database::EAN_13_database(const std::initializer_list<EAN_13>& products) {
	for (EAN_13 product : products)
		_representation[product] = std::bitset<4096>{};
}

int main(int argc, char** argv) {
	// 471-9-5120-0288-9
	EAN_13 product{ {'4','7','1','9','5','1','2','0','0','2','8','8','9'} };
	std::cout << "EAN-13 product: " << product.toString() << std::endl;
	EAN_13 product_{ {'4','7','1','9','5','1','2','0','0','2','8','8','9'} }; // Same data in memory...
	EAN_13_database database({ product,product_ });
	assert(database.size() == 1);

	return 0;
}

